@@ -17,15 +17,34 @@
 
 #@PydevCodeAnalysisIgnoren
 import os
+"""
+This is a check to make sure the sip and the QStrings play nicely in Windows, 
+where the PySide is using the new Python API (Python 3)
+http://www.mail-archive.com/matplotlib-users@lists.sourceforge.net/msg19702.html
+http://stackoverflow.com/questions/1400858/how-to-create-qstring-in-pyqt4
+This could be easily removed when we move to Python 3
+"""
+if os.name == 'nt':
+    import sip
+    sip.setapi('QString', 2)
+    sip.setapi('QVariant', 2)
+    from PyQt4 import QtGui, QtCore, uic
+    from PyQt4.QtCore import *
+
+from subprocess import call
 from manager import SynVecRef
 os.environ['ETS_TOOLKIT'] = 'qt4'
-
+import logging
+FORMAT = '%(levelname)s %(name)s %(lineno)s   %(message)s'
+logging.basicConfig(level=logging.INFO, format=FORMAT)
+logger = logging.getLogger(__name__)
 
 import sys
 sys.path.append(os.path.dirname(__file__)) 
 
-from PyQt4 import QtGui, QtCore, uic
-from PyQt4.QtCore import Qt
+if os.name != 'nt':
+    from PyQt4 import QtGui, QtCore, uic
+    from PyQt4.QtCore import Qt
 
 import numpy as np
 
@@ -33,9 +52,9 @@ import matplotlib as mpl
 if mpl.backends.backend is None: 
     mpl.use('Qt4Agg')
 elif mpl.backends.backend != 'Qt4Agg':
-    print "You must use the Qt4 backend to be able to use  Neuronvisio."
-    print "Check your backend in ~/.matplotlib/matplotlibrc and set it to Qt4Agg"
-
+    message = """You must use the Qt4 backend to be able to use  Neuronvisio. 
+    Check your backend in ~/.matplotlib/matplotlibrc and set it to Qt4Agg"""
+    logger.warning(message)
 mpl.interactive(True)
 
 from neuron import h
@@ -47,6 +66,9 @@ from visio import Visio
 import manager
 import res # icons
 
+# ModelDb
+from modeldb.ModelDB import Models
+
 class Controls():
     """Main class Neuronvisio"""
     def __init__(self):
@@ -93,6 +115,16 @@ class Controls():
         self.ui.actionSave.connect(self.ui.actionSave,
                                    QtCore.SIGNAL("triggered()"),
                                    self.save_hdf)
+        self.ui.tabWidget.connect(self.ui.tabWidget, 
+                                  QtCore.SIGNAL('currentChanged(int)'),
+                                  self.populate_treeview_model)
+        self.ui.tree_models.connect(self.ui.tree_models, 
+                                    QtCore.SIGNAL('itemSelectionChanged ()'),
+                                    self.select_model_treeview)
+        self.ui.load_model_btn.connect(self.ui.load_model_btn, 
+                                    QtCore.SIGNAL('clicked()'),
+                                    self.load_selected_model)
+        
         
         ### Connection with the console
         widgetDic = {'dt' : self.ui.dtSpinBox, 
@@ -106,10 +138,143 @@ class Controls():
         
         ### Manager class 
         self.manager = manager.Manager()
-        self.path_to_hdf = None                    
+        self.path_to_hdf = None
+        self.visio = None
+        self.tab_model_already_populated = False  
         self.ui.show()
         # Start the main event loop.
         #app.exec_()
+        
+        self.AUTHORS = 1
+        self.YEAR = 0
+        self.TITLE = 2
+        self.ID = 3
+        # Dictionary to old the models class for the ModelDb integration
+        self.models = None
+        
+    def populate_treeview_model(self, index):
+        """populate the tree view and the scroll_area when the tab is 
+        activated"""
+        if not self.tab_model_already_populated and index == 3 :
+            
+            self.models =  Models()
+            # Populating the treeview with the dictionary
+            for model_name in self.models.get_model_names():
+                model = self.models.get_model(model_name)
+                model_item = QtGui.QTreeWidgetItem(self.ui.tree_models, 'Models')
+                model_item.setText(self.YEAR, model.get_year())
+                model_item.setText(self.AUTHORS, model.get_authors())
+                model_item.setText(self.TITLE, model.get_title())
+                model_item.setData(self.ID, 0, model.get_id())
+                
+                # tooltip
+                cols = self.ui.tree_models.columnCount()
+                for i in range (cols):
+                    tooltip = model.get_tooltip()
+                    model_item.setToolTip(i, tooltip)
+            
+                
+            #Resizing the column.
+            #self.ui.tree_models.resizeColumnToContents(0)
+            self.ui.tree_models.resizeColumnToContents(self.YEAR)
+            self.ui.tree_models.resizeColumnToContents(self.TITLE)
+            self.ui.tree_models.resizeColumnToContents(self.ID)
+            self.ui.textBrowser_readme.clear()
+            self.ui.textBrowser_readme.insertPlainText("No model selected.")
+            self.tab_model_already_populated = True #we populated only once.
+        
+    def select_model_treeview(self):
+        """Synch the README and the modelOverview with the selected model."""
+        mod = self._retrieve_selected_model()
+        if mod:
+            readme = mod.get_readme_html()
+            overview = mod.get_overview()
+            self.ui.textBrowser_readme.clear()
+            self.ui.textBrowser_readme.insertHtml(readme)            
+            self.ui.textBrowser_model_overview.clear()
+            self.ui.textBrowser_model_overview.insertHtml(overview)
+       
+    def _retrieve_selected_model(self):
+        "Return the model selected in the "
+        items = self.ui.tree_models.selectedItems()
+    
+        if items:
+            selected_item = items[0] #first element
+            model_id = int(selected_item.text(self.ID))
+            models_name = self.models.get_model_names()
+            for name in models_name:
+                mod = self.models.get_model(name)
+                if model_id == mod.get_id():
+                    return mod
+        else:
+            logging.info('No model selected!')
+            return None
+       
+    def load_selected_model(self):
+        "Load the model selected in the treeview."
+        
+        mod = self._retrieve_selected_model()
+        if mod:
+            model_path = mod.download_model()
+            # tooltip
+            cols = self.ui.tree_models.columnCount()
+            items = self.ui.tree_models.selectedItems()
+            model_item = items[0]
+            for i in range (cols):
+                tooltip = mod.get_tooltip()
+                model_item.setToolTip(i, tooltip)
+            self.run_extracted_model(mod)
+
+    # create the command line to compile mod files into nrnmech.dll and launch it. command line is
+    # <cygwin-dir>\bin\bash.exe -c "cd <model-dir>; /usr/bin/sh -c '<nrnhome>/lib/mknrndll.sh <nrnhome>'"
+    def windows_compile_mod_files(self, model_dir):
+        import _winreg
+        k1=_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, "SOFTWARE\\Cygwin\\setup")
+        s1=_winreg.QueryValueEx(k1, 'rootdir')[0]
+        _winreg.CloseKey(k1)
+
+        k2=_winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, "SOFTWARE\\NEURON\\nrn72") 
+        s2=_winreg.QueryValueEx(k2, 'Install_Dir')[0]
+        s2=s2.replace('\\', '/')
+        _winreg.CloseKey(k2)
+
+        cmd=s1+"\\bin\\bash.exe"
+        arg="cd "+model_dir+";/usr/bin/sh -c '" + s2 + "/lib/mknrndll.sh " + s2 + "'"
+        import subprocess
+        subprocess.Popen([cmd, '-c', arg], stdin=subprocess.PIPE).communicate(input="\r\n")
+
+    def run_extracted_model(self, mod):
+        model_dir = mod.get_dir()
+        if os.path.exists(os.path.join (model_dir, 'mosinit.hoc')):
+            old_dir = os.path.abspath(os.getcwd())
+            logger.info("Path changed to %s" %(os.path.abspath(model_dir)))
+            os.chdir(model_dir)
+            try:
+                # If windows
+                if os.name == 'nt':                
+                    self.windows_compile_mod_files('.')
+                    from neuron import h
+                    h.nrn_load_dll('./nrnmech.dll')
+                else: # Anything else.
+                    call(['nrnivmodl'])
+                    import neuron            
+                    neuron.load_mechanisms('./')
+                from neuron import gui # to not freeze neuron gui
+                from neuron import h
+                logger.info("Loading model in %s" %model_dir)
+                h.load_file('mosinit.hoc')
+            except Exception as e:
+                logger.warning("Error running model: "+e.message)
+            logger.info("Path changed back to %s" %old_dir)
+            os.chdir(old_dir)
+        else: 
+            response = """We didn't find any mosinit.hoc . Unfortunately we can't 
+            automatically run the model. Check the README, maybe there is an 
+            hint."""
+            logging.warning(response)
+            path_info = "You can find the extracted model in %s" %model_dir
+            mod.browse()
+            logging.info(path_info)
     
     def load_hdf(self, path_to_hdf=None):
     
@@ -147,11 +312,21 @@ class Controls():
     def launch_visio(self):
         msg = "Plotting..."
         self.ui.statusbar.showMessage(msg, 3500)
-        if not hasattr(self, 'visio'):
-            self.visio = Visio(self.ui.sec_info_label, self.manager)
-            self.visio.draw_model()
+        if self.visio == None:
             
-            self.ui.selected_section.setEnabled(True)
+            # Checking there are sections in the model.
+            i = 0
+            for sec in h.allsec():
+                i += 1
+            
+            if i > 0:
+                self.visio = Visio(self.ui.sec_info_label, self.manager)
+                self.visio.draw_model()
+                self.ui.selected_section.setEnabled(True)
+            else:
+                msg = """No model found, no section created. You need 
+                to have at least one."""
+                logger.warning(msg)
         else:
             #Raise the visio window
             self.visio.container.show()
@@ -168,7 +343,7 @@ class Controls():
         """Set the vm_init from the spin button and prepare the simulator"""
         
         if not self.manager.refs.has_key('VecRef') :
-            print "No vector Created. Create at least one vector to run the simulation"
+            logger.info("No vector Created. Create at least one vector to run the simulation")
             return False
         else:
             v_init = self.ui.vSpinBox.value()
@@ -339,9 +514,12 @@ class Controls():
                           be given by the name (sec.name() from Neuron point 
                           of view. 
         """
-        selection_scalar = self.visio.get_selection_scalar(list_of_sections)
-        self.visio.redraw_color(selection_scalar, 'v')
-        self.visio.update_sections_info(list_of_sections)
+        if self.visio is not None:
+            selection_scalar = self.visio.get_selection_scalar(list_of_sections)
+            self.visio.redraw_color(selection_scalar, 'v')
+            self.visio.update_sections_info(list_of_sections)
+        else:
+            logger.warning("You have to launch the 3D Visio window first!")
     
     def on_animation_time_return_pressed(self):
         "Getting the value from the text"
@@ -364,7 +542,7 @@ class Controls():
             self.sync_visio_3d(time_point_indx)
             self.ui.timelineSlider.setValue(time_point_indx)
         except:
-            print "Value not present in the array."
+            logger.warning("Value not present in the array.")
     
     def on_timeline_value_changed(self):
         """Draw the animation according to the value of the timeline"""
@@ -432,4 +610,4 @@ class Timeloop(QtCore.QThread):
             
                 
             
-            
\ No newline at end of file
+            
